@@ -9,6 +9,7 @@ module Network.WebSockets.Server
     , runServerWith
     , makeListenSocket
     , makePendingConnection
+    , makePendingConnectionFromStream
     ) where
 
 
@@ -83,17 +84,28 @@ runApp :: Socket
        -> ConnectionOptions
        -> ServerApp
        -> IO ()
-runApp socket opts app = do
-    pending <- makePendingConnection socket opts
-    app pending
+runApp socket opts app =
+    bracket
+        (makePendingConnection socket opts)
+        (Stream.close . pendingStream)
+        app
 
 
 --------------------------------------------------------------------------------
--- | Turns a socket, connected to some client, into a 'PendingConnection'.
+-- | Turns a socket, connected to some client, into a 'PendingConnection'. The
+-- 'PendingConnection' should be closed using 'Stream.close' later.
 makePendingConnection
     :: Socket -> ConnectionOptions -> IO PendingConnection
-makePendingConnection sock opts = do
-    stream   <- Stream.makeSocketStream sock
+makePendingConnection socket opts = do
+    stream <- Stream.makeSocketStream socket
+    makePendingConnectionFromStream stream opts
+
+
+-- | More general version of 'makePendingConnection' for 'Stream.Stream'
+-- instead of a 'Socket'.
+makePendingConnectionFromStream
+    :: Stream.Stream -> ConnectionOptions -> IO PendingConnection
+makePendingConnectionFromStream stream opts = do
     -- TODO: we probably want to send a 40x if the request is bad?
     mbRequest <- Stream.parse stream (decodeRequestHead False)
     case mbRequest of
